#! /usr/bin/perl
#
#  Purpose:
#  FlowTracker.cgi creates the FlowTracker web page for inputting
#  selection criteria.
#
#  Description:
#
#  FlowTracker.cgi will create an form for accepting parameters to control
#  the filtering and selection of Netflow data in the generation of a
#  FlowTracker set of graphs.
#
#  Input arguments:
#  Name                 Description
#  -----------------------------------------------------------------------
#  none      
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  07/04/2006      3.0     Original version.
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;

if ($debug_tracker > 0) { open (DEBUG,">$work_directory/DEBUG_TRACKER"); }

# Create a Tracker directory if it doesn't exist

if (!-e $tracker_directory) { 

        mkdir($tracker_directory,0777) || die "cannot mkdir $tracker_directory: $!";   

        print "      The directory for storing Tracking files has been created:\n\n"; 
        print "      <i>$tracker_directory</i>\n\n"; 
        print "      Please ensure this directory has adequate permissions for your\n"; 
        print "      web server process owner (e.g., 'apache') to write into it.\n\n"; 
}

# Copy image into Tracker directory 
      
$copy_command = "cp $cgi_bin_directory/FlowTracker.png $tracker_directory"; 
system($copy_command); 

# Create directory to keep Tracker filter files, if it doesn't exist already
      
if (!-e $filter_directory) {  

        mkdir($filter_directory,0777) || die "Cannot mkdir Tracker filter directory: $filter_directory: $!";  

        print "      The directory for storing Tracking Filter files has been created:\n\n"; 
        print "      <i>$filter_directory</i>\n\n"; 
        print "      Please ensure this directory has adequate permissions for your\n"; 
        print "      web server process owner (e.g., 'apache') to write into it.\n\n"; 
} 

# Create directory to keep Tracker RRDtool files, if it doesn't exist already
      
if (!-e $rrdtool_directory) {  

        mkdir($rrdtool_directory,0777) || die "Cannot mkdir Tracker RRDtool directory: $rrdtool_directory: $!";  

        print "      The directory for storing Tracking RRDtool files has been created:\n\n"; 
        print "      <i>$rrdtool_directory</i>\n\n"; 
        print "      Please ensure this directory has adequate permissions for your\n"; 
        print "      web server process owner (e.g., 'apache') to write into it.\n\n"; 
} 

# Create the header portion of the FlowTracker webpage
 
print "Content-type:text/html\n\n";
print "<html>\n";
print "<head>\n";
print "<title>\n";
print "FlowTracker $version\n";
print "</title>\n";
print "</head>\n";
print "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>\n";
print "<style type=text/css><!-- A { text-decoration:none } --></style>";
print "<center>\n";
print "<img src=$tracker_short/FlowTracker.png></center><b><pre>\n";
print "Filter Criteria:\n\n";
print "<form action=$cgi_bin_short/FlowTracker_Main.cgi method=POST>";

print "     Device: <select name=device_name width=20></b>\n";
foreach $device_name (@devices) {
	print "<option value=$device_name>$device_name\n";
}
print "</select>";

print "       Protocol: <input type=text size=6 name=protocols value=$protocols>     TCP Flag: <input type=text size=10 name=tcp_flags value=$tcp_flags>     TOS Field: <input type=text size=10 name=tos_fields value=$tos_fields>  </b>(e.g., -0x0b/0x0F)<b>\n";
print "\n";
print "  Source IP: <input type=text size=19 name=source_address value=$source_address> </b>(e.g., 192.168.16.0/22)<b> Source Port: <input type=text size=5 name=source_port value=$source_port>    Source Interface: <input type=text size=2 name=source_if value=$source_if>   Source AS: <input type=text size=6 name=source_as value=$source_as>\n";
print "\n";
print "    Dest IP: <input type=text size=19 name=dest_address value=$dest_address> </b>(e.g., 0.0.0.0/0)<b>         Dest Port: <input type=text size=5 name=dest_port value=$dest_port>      Dest Interface: <input type=text size=2 name=dest_if value=$dest_if>     Dest AS: <input type=text size=6 name=dest_as value=$dest_as>\n";
print "\n";
print "<center></b>Note: Multiple field entries, separated by commas, are permitted in the fields above.\n A minus sign (-) will negate an entry (e.g. -1776 for AS, would mean any AS but 1776)<b></center>";
print "<hr size=4>";

# Output FlowTracker specific fields

print "Tracking Parameters:\n\n";
print "  Tracking Set Label: <input type=text size=30 name=tracking_label value=$tracking_label>";
print "\n\n";
print "     General Comment: <input type=text size=90 name=general_comment value=$general_comment>";
print "\n\n";
print "<center>";
print "<input type=hidden name=flow_select value=1>\n";
print "<input type=submit name=action value=\"Establish Tracking\">&nbsp\;<input type=reset value=\"Reset Values\">\n\n";
print "</center>";
print "<hr size=4>";

# List each existing tracking for revision, removal purposes

print "Existing Trackings:\n\n";

@filters = <$filter_directory/*>;
@filters = sort(@filters);

foreach $filter_file (@filters) {
	
        $tracking_file = $filter_file; 
        $tracking_file =~ s#.*/##; 

        open (FILTERS,"<$filter_file"); 
        while (<FILTERS>) { 
                chop;    
                $key = substr($_,0,8);
                if ($key eq " input: ") {

                        ($input,$field,$field_value) = split(/: /);

                        if ($field eq "tracking_label")   { 
				$filter_label = $field_value;

				$filter_file   = "$filter_directory/$tracking_file"; 
				($tracking_prefix,$tracking_suffix) = split(/\./,$tracking_file);
				$html_file     = "$tracker_short/$tracking_prefix/index.html";

				$file_label = $filter_label;
				$file_label =~ s/ /~/g;

				$filter_label_out = $filter_label;
				$length = length($filter_label_out);
	        		for ($i=$length;$i<=80;$i++) { $filter_label_out .= " "; }

				$tracking_link = "<a href=$html_file>$filter_label_out</a>";
				$revise_link   = "<a href=$cgi_bin_short/FlowTracker_Main.cgi?Solicit^$file_label>Revise</a>";
				$remove_link   = "<a href=$cgi_bin_short/FlowTracker_Main.cgi?Remove^$file_label>Remove</a>";

				printf "      %-62s      %-8s      %-8s\n", $tracking_link, $revise_link, $remove_link; }
			else {
				next;
			}
		}
	}
} 

print "</pre>";
print "</form>\n";
print "</body>\n";
print "</html>\n";
