#! /usr/bin/perl
#
#  Purpose:
#  FlowTracker_Main.cgi collects user filter criteria and creates a
#  flow-tools filter file that will be used to create and maintain
#  FlowTracker tracking graphs.
#
#  Description:
#  The script responds to an HTML form from the user in order to collect
#  parameters that will control the collection of netflow data into 
#  RRDtool databases. It also responds to house-keeping chores such as
#  revising or removing exitsing trackings.
#
#  Input arguments (received from the form):
#  Name                 Description
#  -----------------------------------------------------------------------
#  device_name          An identifying name of the device (e.g. router1)
#  source_addresses     Constrain flows examined to these source IP addresses
#  source_ports         Constrain flows examined to these source ports
#  source_ifs           Constrain flows examined to these input interfaces
#  source_ases          Constrain flows examined to these source ASes
#  dest_addresses       Constrain flows examined to these dest. IP addresses
#  dest_ports           Constrain flows examined to these dest. ports
#  dest_ifs             Constrain flows examined to these output interfaces
#  dest_ases            Constrain flows examined to these dest. ASes
#  tos_fields           Constrain flows examined by specified TOS field values 
#  tcp_flags            Constrain flows examined by specified TCP flag values 
#  protocols            Constrain flows examined to these protocols 
#  tracking_label       Label used for identifying Tracking filter and HTML file
#  general_comment      Useful for explaining a particular set of Tracking graphs
#  revision_comment     Comment to explain revision of filtering criteria
#  notate_graphs        Indicates whether user wants to notate graphs with revision
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  07/04/2006      3.0     Original version.
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;

if ($debug_tracker eq "Y") { open (DEBUG,">$work_directory/DEBUG_TRACKER"); }

# Retrieve parameters for a 'Remove' or 'Review/Revise' action

($action,$tracking_label) = split(/\^/,$ENV{'QUERY_STRING'});
if ($action eq "Solicit") { $action = "Solicit Revision"; }
if ($action eq "Revise")  { $action = "Revise Tracking"; }
if ($action eq "Remove")  { $action = "Remove Tracking"; }
$tracking_label =~ s/~/ /g;

print DEBUG "In FlowTracker_Main...\n";

# Otherwise retrieve the form inputs

read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
@pairs = split(/&/, $buffer);
foreach $pair (@pairs) {
    ($name, $value) = split(/=/, $pair);
    $value =~ tr/+/ /;
    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
    $FORM{$name} = $value;
}

# Clean up input

$FORM{source_address} =~ s/\s+//g; $FORM{source_address} =~ s/,/, /g;  
$FORM{source_port}    =~ s/\s+//g; $FORM{source_port}    =~ s/,/, /g;  
$FORM{source_if}      =~ s/\s+//g; $FORM{source_if}      =~ s/,/, /g;  
$FORM{source_as}      =~ s/\s+//g; $FORM{source_as}      =~ s/,/, /g;  
$FORM{dest_address}   =~ s/\s+//g; $FORM{dest_address}   =~ s/,/, /g;  
$FORM{dest_port}      =~ s/\s+//g; $FORM{dest_port}      =~ s/,/, /g;  
$FORM{dest_if}        =~ s/\s+//g; $FORM{dest_if}        =~ s/,/, /g;  
$FORM{dest_as}        =~ s/\s+//g; $FORM{dest_as}        =~ s/,/, /g;  
$FORM{protocols}      =~ s/\s+//g; $FORM{protocols}      =~ s/,/, /g;  
$FORM{tos_fields}     =~ s/\s+//g; $FORM{tos_fields}     =~ s/,/, /g;  
$FORM{tcp_flags}      =~ s/\s+//g; $FORM{tcp_flags}      =~ s/,/, /g; 
	
# Parameters for generating a FlowTracker report

if ($action eq "")         { $action            = $FORM{'action'}; }
if ($tracking_label eq "") { $tracking_label    = $FORM{'tracking_label'}; }

$device_name       = $FORM{'device_name'};
$source_addresses  = $FORM{'source_address'};
$source_ports      = $FORM{'source_port'};
$source_ifs        = $FORM{'source_if'};
$source_ases       = $FORM{'source_as'};
$dest_addresses    = $FORM{'dest_address'};
$dest_ports        = $FORM{'dest_port'};
$dest_ifs          = $FORM{'dest_if'};
$dest_ases         = $FORM{'dest_as'};
$protocols         = $FORM{'protocols'};
$tcp_flags         = $FORM{'tcp_flags'};
$tos_fields        = $FORM{'tos_fields'};
$general_comment   = $FORM{'general_comment'};
$revision_comment  = $FORM{'revision_comment'};
$notate_graphs     = $FORM{'notate_graphs'};
$flow_select       = $FORM{'flow_select'};

$FORM{'start_date'} = "01/01/2000";
$FORM{'start_time'} = "00:00:00";
$FORM{'end_date'}   = "01/01/2000";
$FORM{'end_time'}   = "00:00:00";

$tracking_file = $tracking_label;
$tracking_file =~ s/^\s+//;
$tracking_file =~ s/\s+$//;
$tracking_file =~ s/\&/-/g;
$tracking_file =~ s/\//-/g;
$tracking_file =~ s/\(/-/g;
$tracking_file =~ s/\)/-/g;
$tracking_file =~ s/\./-/g;
$tracking_file =~ s/\s+/_/g;
$tracking_file =~ tr/[A-Z]/[a-z]/;

$filter_file    = $filter_directory ."/". $tracking_file .".fil";
$rrdtool_file   = $rrdtool_directory ."/". $tracking_file .".rrd";

$html_directory = $tracker_directory ."/". $tracking_file;
$html_file      = $html_directory ."/index.html";
	
if ($tracking_label eq "") {
	print "<br>\n<b>    Blank Tracking label. Please fill this in.\n</b>";
}

if ($debug_tracker eq "Y") {
	print DEBUG "        action: $action\n";
	print DEBUG "tracking_label: $tracking_label\n";
	print DEBUG " tracking_file: $tracking_file\n";
}

if ($action eq "Establish Tracking") {

	# Start the web page output
	
	print "Content-type:text/html\n\n";
	print "<head>";
	print "<style type=text/css><!-- A { text-decoration:none } --></style>";
	print "</head>";
	print "<body bgcolor=$bg_color>";
	print "<title>FlowTracker $version Setup Results</title>";
	print "<pre>\n";
	print "<b>";
	print "\n\n";

	# Make the Filter directory if it doesn't exist
	
	if (!-e $filter_directory) {

	        mkdir($filter_directory,0777) || die "cannot mkdir $filter_directory: $!";  
		chmod (0777,"$filter_directory");

		print "      The directory for storing Tracking Filter files has been created:\n\n";
		print "      <i>$filter_directory</i>\n\n";
		print "      Please ensure this directory has adequate permissions for your\n";
		print "      web server process owner (e.g., 'apache') to write into it.\n\n";
	}
	
	# Make the RRDtool directory if it doesn't exist
	       
	if (!-e $rrdtool_directory) {   
	      
	        mkdir($rrdtool_directory,0777) || die "Cannot mkdir Tracking RRDtool directory: $rrdtool_directory: $!";   
		chmod (0777,"$rrdtool_directory");
	      
	        print "      The directory for storing Tracking RRDtool files has been created:\n\n";  
	        print "      <i>$rrdtool_directory</i>\n\n";  
	        print "      Please ensure this directory has adequate permissions for your\n";  
	        print "      web server process owner (e.g., 'apache') to write into it.\n\n";  
	}     

	# Compare Tracking Label to existing Trackings
	
	while ($existing_filter = <$filter_directory/*>) {

		if ($filter_file eq $existing_filter) { 

			$match = 1;

			print "      An existing Filter file has been found with the same Tracking label.\n\n";
			print "            Tracking Label: <i>$tracking_label</i>\n";
			print "      Tracking Filter File: <i>$filter_file</i>\n\n";
			print "      If you wish to replace the existing Tracking, please remove the existing\n"; 
			print "      Filter file from your Filter directory, and resubmit the FlowTracker form.\n";
			print "\n";
			print "      Or, if you wish to continue an existing Tracking, but with new filtering\n";
			print "      criteria, return to the previous page and select the \'Revise\' option from\n";
			print "      the entry in the list of 'Existing Trackings\' at the bottom.\n\n";
			print "      <a href=$cgi_bin_short/FlowTracker.cgi>RETURN</a>\n";

			exit;
		}
	}
	
	if (!($match)) {
	
		if ($debug_tracker eq "Y") { print DEBUG "creating new filter file: $filter_file\n"; }

		# Create the filter to match the input specifications
	
		create_filter_file (%FORM, $filter_file);

		if ($debug_tracker eq "Y") { print DEBUG "created filter file: $filter_file\n"; }
	
		# Create the RRDtool database for this Tracking

		if ($debug_tracker eq "Y") { print DEBUG "creating new RRD file: $rrdtool_file\n"; }

		$start_rrd = time - (40 * 60);

                $rrdtool_command =     "$rrdtool_bin_directory/rrdtool create $rrdtool_file ". 
                                        "--step 300 ".
                                        "--start $start_rrd ".
                                        "DS:flowbits:GAUGE:600:U:U ". 
                                        "RRA:AVERAGE:0.5:1:600 ". 
                                        "RRA:AVERAGE:0.5:6:700 ". 
                                        "RRA:AVERAGE:0.5:24:775 ". 
                                        "RRA:AVERAGE:0.5:288:797 ". 
                                        "RRA:MAX:0.5:1:600 ". 
                                        "RRA:MAX:0.5:6:700 ". 
                                        "RRA:MAX:0.5:24:775 ". 
                                        "RRA:MAX:0.5:288:797"; 

        	system($rrdtool_command); 
		chmod (0777,"$rrdtool_file");

		print "      You have successfully setup a new Tracking. The graphs for this\n";
		print "      new Tracking will appear after the next FlowTracker_Grapher run.\n\n";
		print "            Tracking Label: <i>$tracking_label</i>\n";
		print "               Filter File: <i>$filter_file</i>\n";
		print "          RRDtool Database: <i>$rrdtool_file</i>\n";
		print "            HTML Directory: <i>$html_directory</i>\n\n";

		print "      <a href=$cgi_bin_short/FlowTracker.cgi>RETURN</a>\n";
		print "</b>\n";
		print "</pre>\n"; 
		print "</body>\n"; 
		print "</html>\n"; 
	}
	
	open (FILTER,">>$filter_file");
	print FILTER "\n\n";
	print FILTER " input: device_name: $device_name\n";
	print FILTER " input: tracking_label: $tracking_label\n";
	print FILTER " input: general_comment: $general_comment\n";
	print FILTER " input: source_addresses: $source_addresses\n";
	print FILTER " input: source_ports: $source_ports\n";
	print FILTER " input: source_ifs: $source_ifs\n";
	print FILTER " input: source_ases: $source_ases\n";
	print FILTER " input: dest_addresses: $dest_addresses\n";
	print FILTER " input: dest_ports: $dest_ports\n";
	print FILTER " input: dest_ifs: $dest_ifs\n";
	print FILTER " input: dest_ases: $dest_ases\n";
	print FILTER " input: protocols: $protocols\n";
	print FILTER " input: tos_fields: $tos_fields\n";
	print FILTER " input: tcp_flags: $tcp_flags\n";

	&create_FlowTracker_html;

}

elsif ($action eq "Remove Tracking") {
	
	# Start the web page output
	
	print "Content-type:text/html\n\n";
	print "<head>";
	print "<style type=text/css><!-- A { text-decoration:none } --></style>";
	print "</head>";
	print "<body bgcolor=$bg_color>";
	print "<title>FlowTracker Tracking Removal</title>";
	print "<pre>\n";
	print "<b>";
	print "\n\n";

	if (!-e $work_directory) {   
	        mkdir($work_directory,0777) || die "Cannot mkdir Work directory: $work_directory: $!";   
		chmod (0777,"$work_directory");
	}
	      
	# Save graphs to work directory, and remove directory and contents

	$cp_command = "cp -R $html_directory $work_directory";
	system($cp_command);

	$rmdir_command = "rm -r $html_directory";
	system($rmdir_command);

	$html_file = $html_directory;
	$html_file =~ s#.*/##;

	# Save filter to work directory, and remove file

	$cp_command = "cp  $filter_file $work_directory/$html_file";
	system($cp_command);

	$rm_command = "rm -f $filter_file";
	system($rm_command);

	# Save rrd file to work directory, and remove file

	$cp_command = "cp  $rrdtool_file $work_directory/$html_file";
	system($cp_command);

	$rm_command = "rm -f $rrdtool_file";
	system($rm_command);

	$filter_file  =~ s#.*/##;
	$rrdtool_file =~ s#.*/##;

        print "      All files and directories associated with:\n\n"; 
        print "      <i>$tracking_label</i>\n\n"; 
	print "      have been moved to your Work directory, under subdirectory:\n\n";
	print "      $work_directory/$html_file\n\n";
        print "      If necessary, you can recreate this Tracking by moving the\n";
        print "      files back to the operational directories. That would be:\n\n";
	print "      1) Move <font color=$vlink_color>$filter_file</font> back to $filter_directory\n";
	print "      2) Move <font color=$vlink_color>$rrdtool_file</font> back to $rrdtool_directory\n";
	print "      3) Move <font color=$vlink_color>$html_file</font> directory and contents back to $tracker_directory\n\n\n";

	print "      <a href=$cgi_bin_short/FlowTracker.cgi>RETURN</a>\n";
	print "</b>\n";
	print "</pre>\n"; 
	print "</body>\n"; 
	print "</html>\n"; 
}

elsif ($action eq "Solicit Revision") {

	# Start the web page output
	
	print "Content-type:text/html\n\n";
	print "<head>\n";
	print "<title>FlowTracker Revision Inputs</title>\n";
	print "</head>\n";
	print "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>\n";
	print "<style type=text/css><!-- A { text-decoration:none } --></style>";
	print "<center>\n";
	print "<a href=$cgi_bin_short/FlowTracker.cgi><img src=$tracker_short/FlowTracker.png border=0></a></center><b><pre>\n";

	# Retrieve form input for Tracking to be modified

	open (FILTER,"<$filter_file");
	while (<FILTER>) {
		chop;
		$key = substr($_,0,8);
	        if ($key eq " input: ") { 
			($input,$field,$field_value) = split(/: /);	
			if ($field eq "device_name")      { $revise_device_name = $field_value; }
			if ($field eq "source_addresses") { $source_address = $field_value; }
			if ($field eq "source_ports")     { $source_port = $field_value; }
			if ($field eq "source_ifs")       { $source_if = $field_value; }
			if ($field eq "source_ases")      { $source_as = $field_value; }
			if ($field eq "dest_addresses")   { $dest_address = $field_value; }
			if ($field eq "dest_ports")       { $dest_port = $field_value; }
			if ($field eq "dest_ifs")         { $dest_if = $field_value; }
			if ($field eq "dest_ases")        { $dest_as = $field_value; }
			if ($field eq "protocols")        { $protocols = $field_value; }
			if ($field eq "tos_fields")       { $tos_fields = $field_value; }
			if ($field eq "tcp_flags")        { $tcp_flags = $field_value; }
			if ($field eq "tracking_label")   { $tracking_label = $field_value; }
			if ($field eq "general_comment")  { $general_comment = $field_value; }
		}
	}
	close (FILTER);

	print "Filter Criteria:\n\n";
	print "<form action=$cgi_bin_short/FlowTracker_Main.cgi method=POST>";
	
	print "     Device: <SELECT name=device_name width=20></b>\n";
	foreach $device_name (@devices) {
		print "<OPTION value=$device_name>$device_name\n";
	}
	print "<OPTION selected>$revise_device_name\n";
	print "</SELECT>";
	
	print "       Protocol: <input type=text size=6 name=protocols value=\"$protocols\">     TCP Flag: <input type=text size=10 name=tcp_flags value=\"$tcp_flags\">     TOS Field: <input type=text size=10 name=tos_fields value=\"$tos_fields\">  </b>(e.g., -0x0b/0x0F)<b>\n";
	print "\n";
	print "  Source IP: <input type=text size=19 name=source_address value=\"$source_address\"> </b>(e.g., 192.168.16.0/22)<b> Source Port: <input type=text size=5 name=source_port value=\"$source_port\">    Source Interface: <input type=text size=2 name=source_if value=\"$source_if\">   Source AS: <input type=text size=6 name=source_as value=\"$source_as\">\n";
	print "\n";
	print "    Dest IP: <input type=text size=19 name=dest_address value=\"$dest_address\"> </b>(e.g., 0.0.0.0/0)<b>         Dest Port: <input type=text size=5 name=dest_port value=\"$dest_port\">      Dest Interface: <input type=text size=2 name=dest_if value=\"$dest_if\">     Dest AS: <input type=text size=6 name=dest_as value=\"$dest_as\">\n";
	print "\n";
	print "<center></b>Note: Multiple field entries, separated by commas, are permitted in the fields above.\n A minus sign (-) will negate an entry (e.g. -1776 for AS, would mean any AS but 1776)<b></center>";
	print "<HR size=4>";
	print "Tracking Parameters:\n\n";
	print "  Tracking Set Label: $tracking_label";
	print "\n\n";
	print "     General Comment: <input type=text size=90 name=general_comment value=\"$general_comment\">";
	print "\n\n";
	print "    Revision Comment: <input type=text size=90 name=revision_comment value=\"$revision_comment\">";
	print "\n\n";
	print "       Notate Graphs: <select name=notate_graphs width=4></b>\n";  
	print "    <option selected value=N>N\n";  
	print "    <option value=Y>Y\n";  
	print "    </select></b> (To notate graphs with time and description of revision)"; 
	print "\n\n";
	print "<center>";
	print "<input type=hidden name=tracking_label value=\"$tracking_label\">\n";
	print "<input type=hidden name=flow_select value=1>\n";
	print "<input type=submit name=action value=\"Revise Tracking\">&nbsp\;<input type=reset value=\"Reset Values\">\n\n";
	print "</center>";
	print "<hr size=4>";
	
	print "</pre>";
	print "</form>\n";
	print "</body>\n";
	print "</html>\n";
}

elsif ($action eq "Revise Tracking") {

	# Start the web page output
	
	print "Content-type:text/html\n\n";
	print "<head>";
	print "<style type=text/css><!-- A { text-decoration:none } --></style>";
	print "</head>";
	print "<body bgcolor=$bg_color>";
	print "<title>FlowTracker Revision Results</title>";
	print "<pre>\n";
	print "<b>";
	print "\n\n";

	print "  Old Filtering Criteria:\n\n";

	open  (OLD_FILTER,"<$filter_file");
	while (<OLD_FILTER>) { 
		chop;
		$key = substr($_,0,8);
		if ($key eq " input: ") {
			($input,$field,$field_value) = split(/: /);	
			if ($field eq "revision") { 
				push (@revisions,$field_value);
				($old_notate_graphs,$revision_date_out,$old_revision_comment) = split(/\|/,$field_value);
				$revision_date_out =~ s/\^/:/g;
				print "   input: revision: $old_notate_graphs | $revision_date_out | $old_revision_comment\n"; }
			else {
				print "  $_\n";
			}
		}
	}
	close (OLD_FILTER);

	create_filter_file (%FORM, $filter_file);

	if ($debug_tracker eq "Y") { print DEBUG "created filter file: $filter_file\n"; }
	
	open (NEW_FILTER,">>$filter_file");

	print NEW_FILTER "\n\n";
	print NEW_FILTER " input: device_name: $device_name\n";
	print NEW_FILTER " input: tracking_label: $tracking_label\n";
	print NEW_FILTER " input: general_comment: $general_comment\n";
	print NEW_FILTER " input: source_addresses: $source_addresses\n";
	print NEW_FILTER " input: source_ports: $source_ports\n";
	print NEW_FILTER " input: source_ifs: $source_ifs\n";
	print NEW_FILTER " input: source_ases: $source_ases\n";
	print NEW_FILTER " input: dest_addresses: $dest_addresses\n";
	print NEW_FILTER " input: dest_ports: $dest_ports\n";
	print NEW_FILTER " input: dest_ifs: $dest_ifs\n";
	print NEW_FILTER " input: dest_ases: $dest_ases\n";
	print NEW_FILTER " input: protocols: $protocols\n";
	print NEW_FILTER " input: tos_fields: $tos_fields\n";
	print NEW_FILTER " input: tcp_flags: $tcp_flags\n";

	foreach $revision (@revisions) {
		print NEW_FILTER " input: revision: $revision\n";
	}

	# Need to determine revision date/time as it will apply to next collection

        open (INFO,">$work_directory/FlowTracker_Main_info"); 
        $rrd_info_command = "$rrdtool_bin_directory/rrdtool info $rrdtool_file > $work_directory/FlowTracker_Main_info";
        system($rrd_info_command); 

        open (INFO,"<$work_directory/FlowTracker_Main_info"); 
        while (<INFO>) { 
                chop;    
                $lead = substr($_,0,11); 
                if ($lead eq "last_update") {  
                        ($lead,$last_update) = split(/ = /); 
                }        
        }        
        close (INFO);  

	print NEW_FILTER " input: revision: $notate_graphs|$last_update|$revision_comment\n";

	close (NEW_FILTER);

	print "\n\n  New Filtering Criteria:\n\n";

	open  (NEW_FILTER,"<$filter_file");
	while (<NEW_FILTER>) { 
		$key = substr($_,0,8);
		if ($key eq " input: ") {
			($input,$field,$field_value) = split(/: /);	
			if ($field eq "revision") { 
				($new_notate_graphs,$revision_date,$new_revision_comment) = split(/\|/,$field_value);
				print "   input: revision: $new_notate_graphs | $revision_date | $new_revision_comment"; }
			else {
				print "  $_";
			}
		}
	}
	close (NEW_FILTER);

	&create_FlowTracker_html;

	print "\n\n  <a href=$cgi_bin_short/FlowTracker.cgi>RETURN</a>\n";
}

sub create_FlowTracker_html {

	# Create the web page to hold the RRDtool graphs
	
	if (!-e $html_directory) {  
	        mkdir($html_directory,0777) || die "cannot mkdir $html_directory: $!";  
	} 
	
	chmod (0777,"$html_directory");

	$copy_command = "cp $cgi_bin_directory/FlowTracker.png $html_directory";  
	system($copy_command);
	$copy_command = "cp $cgi_bin_directory/FlowTracker_Links.png $html_directory";  
	system($copy_command);

	if ($debug_tracker eq "Y") { print DEBUG "creating html file: $html_file\n"; }

        open(HTML,">$html_file") || die "cannot open HTML file for write: $html_file";

        print HTML "<html>\n"; 
        print HTML "<head>\n"; 
        print HTML "<title>FlowTracker $version: $tracking_label</title>\n"; 
        print HTML "</head>\n"; 
        print HTML "<body bgcolor=$bg_color TEXT=$text_color LINK=$link_color VLINK=$vlink_color>\n"; 
        print HTML "\n\n";  
        print HTML "<font face=arial>"; 
        print HTML "<table>\n"; 
        print HTML "<tr>\n"; 
        print HTML "<td align=middle align=center>\n"; 
        print HTML "<img src=FlowTracker_Links.png border=0 usemap=#FlowTracker_Map>&nbsp&nbsp&nbsp<font face=arial size=+2>$tracking_label</font>\n"; 
        print HTML "</table>\n"; 
        print HTML "<pre>"; 

	print_formatted_parameters(%FORM, $html_file);
	print HTML "\n"; 

        # Format the comment for clean line-breaks 
     
        $length_comment = length($general_comment); 

        if ($length_comment <= 76) { 
                print HTML "    Comments: $general_comment"; } 
        else {   
                $general_comment_1 = substr($general_comment,0,76); 
                $last_blank_1 = rindex($general_comment_1," "); 
                $general_comment_1 = substr($general_comment,0,$last_blank_1); 
                $length_printed_1 = length($general_comment_1) + 1;
     
                $first_letter_2 = $last_blank_1 + 1; 
                $general_comment_2 = substr($general_comment,$first_letter_2,76); 
                $length_printed = length($general_comment_2) + 1;
                $length_remaining = $length_comment - $length_printed_1 - $length_printed_2;
     
                if ($length_remaining > 76) { 
                        $last_blank_2 = rindex($general_comment_2," "); 
                        $general_comment_2 = substr($general_comment,$first_letter_2,$last_blank_2); 
                        $first_letter_3 = $last_blank_1 + $last_blank_2 + 2; 
                        $general_comment_3 = substr($general_comment,$first_letter_3,76); 
                	$length_printed_3 = length($general_comment_3) + 1;
                	$length_remaining = $length_comment - $length_printed_1 - $length_printed_2 - $length_printed_3;
                }        
     
                if ($length_remaining > 76) { 
                        $last_blank_3 = rindex($general_comment_3," "); 
                        $general_comment_3 = substr($general_comment,$first_letter_3,$last_blank_3); 
                        $first_letter_4 = $last_blank_1 + $last_blank_2 + $last_blank_3 + 3; 
                        $general_comment_4 = substr($general_comment,$first_letter_4,76); 
                }        
     
                if ($general_comment_1 ne "") { print HTML "    Comments: $general_comment_1"; } 
                if ($general_comment_2 ne "") { print HTML "\n              $general_comment_2"; } 
                if ($general_comment_3 ne "") { print HTML "\n              $general_comment_3"; } 
                if ($general_comment_4 ne "") { print HTML "\n              $general_comment_4"; } 
        }        
     
        print HTML "<hr size=4 align=left width=$hr_width><br>"; 
        print HTML "  <img src=$tracker_short/$tracking_file/daily.png>\n";
        print HTML "<br><br><hr size=4 align=left width=$hr_width><br>";
        print HTML "  <img src=$tracker_short/$tracking_file/weekly.png>\n";
        print HTML "<br><br><hr size=4 align=left width=$hr_width><br>";
        print HTML "  <img src=$tracker_short/$tracking_file/monthly.png>\n";
        print HTML "<br><br><hr size=4 align=left width=$hr_width><br>";
        print HTML "  <img src=$tracker_short/$tracking_file/yearly.png>\n";
        print HTML "</pre>\n";
        print HTML "</font>\n";
	print HTML "<map name=FlowTracker_Map>\n";
	print HTML "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowTracker.cgi?preset=$filter_file>\n";
	print HTML "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowTracker.cgi?preset=$filter_file>\n";
	print HTML "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowViewer.cgi?preset=$filter_file>\n";
	print HTML "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowGrapher.cgi?preset=$filter_file>\n";
	print HTML "</map>\n";
        print HTML "</body>\n";
        print HTML "</html>\n";

	close (HTML);
}

sub print_error { 

        my ($error_text) = @_; 

	$rm_command = "/bin/rm $filter_file";
	system($rm_command);

        print "<head>"; 
        print "</head>"; 
        print "<body bgcolor=$bg_color>"; 
        print "<title>FlowTracker Error</title>"; 
        print "<center>\n"; 
        print "<h3>$error_text</h3><br>"; 
	print " Use browser \"bask\" key to preserve inputs for modification<br>";
        print "<br>\n"; 
        print "<form action=$FlowViewer_service://$FlowViewer_server/$cgi_bin_short/FlowTracker.cgi method=link>"; 
        print "<input type=submit value=\"New FlowTracker Inputs\"></form>"; 
     
        exit;    

}    
