#! /usr/bin/perl
#
#  Purpose:
#  FlowViewer.cgi creates the FlowViewer web page for inputting
#  selection criteria.
#
#  Description:
#
#  FlowViewer.cgi will create an form for accepting parameters
#  to control the filtering and selection of Netflow data.
#
#  Input arguments:
#  Name                 Description
#  -----------------------------------------------------------------------
#  none      
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  07/04/2005      1.0     Original version.
#  J. Loiacono  01/01/2006      2.0     New filter parameters
#  J. Loiacono  01/26/2006      2.2     Added flow_select option
#  J. Loiacono  07/04/2006      3.0     Replaces create_FV_webpage
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;

# Copy image into Reports directory

$copy_command = "cp $cgi_bin_directory/FlowViewer.png $reports_directory";
system($copy_command);

# Set up start and end times according to configurable start and end offsets  
      
($sec,$min,$hr,$date,$mnth,$yr,$day,$yr_date,$DST) = localtime(time-$start_offset); 
$mnth += 1; 
$yr   += 1900; 
$min_delta = $min % 5; 
$min = $min - $min_delta; 
if (length $min < 2) { $min = "0" . $min; }  
$start_date = $mnth ."/". $date ."/". $yr; 
if ($use_even_hours eq "Y") {
	$start_time = $hr .":00:00"; }
else {
	$start_time = $hr .":". $min .":00"; }
      
($sec,$min,$hr,$date,$mnth,$yr,$day,$yr_date,$DST) = localtime(time-$end_offset); 
$mnth += 1; 
$yr   += 1900; 
$min_delta = $min % 5; 
$min = $min - $min_delta; 
if (length $min < 2) { $min = "0" . $min; }  
$end_date = $mnth ."/". $date ."/". $yr;
if ($use_even_hours eq "Y") {
	$end_time = $hr .":00:00"; }
else {
	$end_time = $hr .":". $min .":00"; }

# Create the header portion of the FlowViewer webpage

print "Content-type:text/html\n\n";
print "<html>\n";
print "<head>\n";
print "<title>\n";
print "FlowViewer $version\n";
print "</title>\n";
print "</head>\n";
print "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>";
print "<center>";
print "<img src=$reports_short/FlowViewer.png></center><b><pre>";

# If a user has called this program from FlowTracker, pre-fill the input fields

$query_string = $ENV{'QUERY_STRING'}; 
      
($preset, $filter_file) = split(/=/,$query_string) ; 
$preset      =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/ge ; 
$filter_file =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/ge ; 
      
if ($preset ne "") { 
        open (FILTER,"<$filter_file"); 
        while (<FILTER>) { 
                chop;    
                $key = substr($_,0,8); 
                if ($key eq " input: ") { 
                        ($input,$field,$field_value) = split(/: /); 
                        if ($field eq "device_name")      { $selected_device_name = $field_value; } 
                        if ($field eq "source_addresses") { $source_address = $field_value; } 
                        if ($field eq "source_ports")     { $source_port = $field_value; } 
                        if ($field eq "source_ifs")       { $source_if = $field_value; } 
                        if ($field eq "source_ases")      { $source_as = $field_value; } 
                        if ($field eq "dest_addresses")   { $dest_address = $field_value; } 
                        if ($field eq "dest_ports")       { $dest_port = $field_value; } 
                        if ($field eq "dest_ifs")         { $dest_if = $field_value; } 
                        if ($field eq "dest_ases")        { $dest_as = $field_value; } 
                        if ($field eq "protocols")        { $protocols = $field_value; } 
                        if ($field eq "tos_fields")       { $tos_fields = $field_value; } 
                        if ($field eq "tcp_flags")        { $tcp_flags = $field_value; } 
                        if ($field eq "tracking_label")   { $tracking_label = $field_value; } 
                        if ($field eq "general_comment")  { $general_comment = $field_value; } 
                }        
        }        
        close (FILTER); 
}   

print "Filter Criteria:\n\n";
print "<form action=$cgi_bin_short/FlowViewer_Main.cgi method=POST>";

if ($devices[0] ne "") {
	print "     Device: <select name=device_name width=20></b>\n";
        foreach $device_name (@devices) {  
                if ($device_name eq $selected_device_name) { 
                        print "<option selected value=$device_name>$device_name\n"; } 
                else {   
                        print "<option value=$device_name>$device_name\n"; 
                }        
        }
	print "</select>";
	print "\n\n";
}

$cutoff_lines = 100;
$sort_field   = 4;

print " Start Date: <input type=text size=10 name=start_date value=$start_date></b> (mm/dd/yyyy)<b>    Start Time: <input type=text size=10 name=start_time value=$start_time>  </b>(hh:mm:ss)<b>    TOS Fields: <input type=text size=10 name=tos_fields value=\"$tos_fields\">  </b>(e.g., -0x0b/0x0F)<b>\n";
print "\n";
print "   End Date: <input type=text size=10 name=end_date value=$end_date></b> (mm/dd/yyyy)<b>      End Time: <input type=text size=10 name=end_time value=$end_time>  </b>(hh:mm:ss)<b>     TCP Flags: <input type=text size=10 name=tcp_flags value=\"$tcp_flags\">   Protocols: <input type=text size=6 name=protocols value=\"$protocols\">\n";
print "\n";
print "  Source IP: <input type=text size=19 name=source_address value=\"$source_address\"> </b>(e.g., 192.168.16.0/22)<b> Source Port: <input type=text size=5 name=source_port value=\"$source_port\">    Source Interface: <input type=text size=2 name=source_if value=\"$source_if\">   Source AS: <input type=text size=6 name=source_as value=\"$source_as\">\n";
print "\n";
print "    Dest IP: <input type=text size=19 name=dest_address value=\"$dest_address\"> </b>(e.g., 0.0.0.0/0)<b>         Dest Port: <input type=text size=5 name=dest_port value=\"$dest_port\">      Dest Interface: <input type=text size=2 name=dest_if value=\"$dest_if\">     Dest AS: <input type=text size=6 name=dest_as value=\"$dest_as\">\n";
print "\n";
print "<center></b>Note: Multiple field entries, separated by commas, are permitted in the fields above.\n A minus sign (-) will negate an entry (e.g. -1776 for AS, would mean any AS but 1776)<b></center>";
print "<hr size=4>";
print "Reporting Parameters:\n\n";
print " Statistics: <select name=stat_report width=16></b>\n";
print "<option value=0 selected>Statistics Reports\n";
print "<option value=99>Summary\n";
print "<option value=5>UDP/TCP Destination Port\n";
print "<option value=6>UDP/TCP Source Port\n";
print "<option value=7>UDP/TCP Port\n";
print "<option value=8>Destination IP\n";
print "<option value=9>Source IP\n";
print "<option value=10>Source/Destination IP\n";
print "<option value=11>Source or Destination IP\n";
print "<option value=12>IP Protocol\n";
print "<option value=17>Input Interface\n";
print "<option value=18>Output Interface\n";
print "<option value=23>Input/Output Interface\n";
print "<option value=19>Source AS\n";
print "<option value=20>Destination AS\n";
print "<option value=21>Source/Destination AS\n";
print "<option value=22>IP ToS\n";
print "<option value=24>Source Prefix\n";
print "<option value=25>Destination Prefix\n";
print "<option value=26>Source/Destination Prefix\n";
print "</select>";
print "  Printed: <select name=print_report width=16></b>\n";
print "<option value=0 selected>Print Reports\n";
print "<option value=1>Flow Times\n";
print "<option value=4>AS Numbers\n";
print "<option value=5>132 Columns\n";
print "<option value=9>1 Line with Tags\n";
print "<option value=10>AS Aggregation\n";
print "<option value=11>Protocol Port Aggregation\n";
print "<option value=12>Source Prefix Aggregation\n";
print "<option value=13>Destination Prefix Aggregation\n";
print "<option value=14>Prefix Aggregation\n";
print "<option value=24>Full (Catalyst)\n";
print "</select>";
print "  Include Flow if: <select name=flow_select width=16></b>\n";
print "<option value=1 selected>Any Part in Specified Time Span\n";
print "<option value=2>End Time in Specified Time Span\n";
print "<option value=3>Start Time in Specified Time Span\n";
print "<option value=4>Entirely in Specified Time Span\n";
print "</select>";
print "\n\n";
print " Sort Field: <input type=text size=3 name=sort_field value=$sort_field>";
print "   Cutoff Lines: <input type=text size=3 name=cutoff_lines value=$cutoff_lines>";
print "   Cutoff Octets: <input type=text size=13 name=cutoff_octets value=$cutoff_octets>";
print "  Resolve Addresses: <select name=resolve_addresses width=4></b>\n";
print "    <option selected value=Y>Y\n";
print "    <option value=N>N\n";
print "    </select>";
print "\n\n";
print "<hr size=4>";
print "\n";
print "<center>";
print "<input type=submit name=action value=\"Generate Report\">&nbsp\;<input type=reset value=\"Reset Values\">\n";
print "</center>";
print "</pre>";
print "</form>\n";
print "</body>\n";
print "</html>\n";
