#! /usr/bin/perl
#
#  Purpose:  
#  FlowViewer_Configuration.pm holds global variables for the FlowViewer,
#  FlowGrapher, and FlowTracker NetFlow analysis tools.
#
#  Description:
#  Various parameters used to configure the system to a local environment.
#
#  Input arguments:
#  Name                 Description
#  -----------------------------------------------------------------------
#  None
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  07/04/2005      1.0     Original version.
#  J. Loiacono  01/01/2006      2.0     Flowgrapher, new functions, speed
#  J. Loiacono  01/16/2006      2.1     Introduced $flow_file_length
#  J. Loiacono  07/04/2006      3.0     Added parameters for Tracking, others
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
# Path variable

$ENV{PATH}               .= ':/usr/local/bin:/usr/sbin';

# Server

$FlowViewer_server       = "192.168.100.1";     # (IP address or hostname)

# Service

$FlowViewer_service      = "https";           # (http, or https)

# Directories and Files:

$reports_directory       = "/htp/htdocs/FlowViewer";
$reports_short           = "/FlowViewer";
$graphs_directory        = "/htp/htdocs/FlowGrapher";
$graphs_short            = "/FlowGrapher";
$tracker_directory       = "/htp/htdocs/FlowTracker";
$tracker_short           = "/FlowTracker";
$cgi_bin_directory       = "/htp/cgi-bin/FlowViewer_3.0";
$cgi_bin_short           = "/cgi-bin/FlowViewer_3.0";
$work_directory          = "/tmp";
$names_directory         = "/tmp";
$filter_directory        = "/htp/cgi-bin/Flow_Temp/FlowTracker_Filters";
$rrdtool_directory       = "/htp/cgi-bin/Flow_Temp/FlowTracker_RRDtool";

$flow_data_directory     = "/htp/flows";
$flow_bin_directory      = "/usr/bin";
$rrdtool_bin_directory   = "/usr/local/rrdtool-1.2.12/bin";

$tracker_webpage         = "index.html";
$trackings_title         = "Your System Here";

# General parameters

$version                 = "3.0";
@devices                 = ("router_1","router_2","router_3","router_4"); 
$flow_capture_interval   = (30 * 60);
$flow_file_length        = (15 * 60);
$start_offset            = (90 * 60);  # e.g., 90 minutes ago
$end_offset              = (30 * 60);  # e.g., 30 minutes ago
$use_even_hours          = "Y";
$N                       = 3;
$use_NDBM                = "N";
$maximum_days            = "91";

# Debug Parameters 

$debug_viewer            = "N";
$debug_grapher           = "N";
$debug_tracker           = "N";
$debug_files             = "N";

# Graphing parameters
 
$transparent        = "0";   
$x_ticks            = "T";
$long_ticks         = "T";
$skip_undef         = "T";
$graph_height       = 250;
$graph_width        = 600;
$graph_obj_height   = 330;
$graph_obj_width    = 600;
$t_margin           = 10;
$b_obj_margin       = 80;
$b_margin           = 20;
$l_margin           = 10;
$r_margin           = 20;
$bgclr              = "white";
$borderclrs         = "black";
$boxclr             = "white";
$fgclr              = "gray90";
$labelclr           = "black";
$axislabelclr       = "black";
$legendclr          = "black";
$valuesclr          = "black";
$textclr            = "black";
$x_axis_font        = "('arial', 16)";

# Tracking parameters
 
$log_directory      = "/htp/cgi-bin/FlowViewer_3.0";
$log_collector      = "Y";
$log_grapher        = "Y";
$collector_bucket   = 5;
$collection_offset  = 1800;
$collection_period  = 300;
$graphing_period    = 300;

$rrd_area           = "FFE0C0";   
$rrd_line           = "000000";   
$rrd_width          = 600;
$rrd_height         = 150;
$rrd_font           = "000000AA";
$rrd_back           = "FFFFFF";
$rrd_canvas         = "FFFFFF";
$rrd_grid           = "CCCCCC88";
$rrd_mgrid          = "CCCCCC88";
$rrd_frame          = "FFFFFF";
$rrd_shadea         = "FFFFFF";
$rrd_shadeb         = "FFFFFF";
$rrd_thick          = 0.3;
$rrd_lower_limit    = 0;
$rrd_slope_mode     = "";      # $rrd_slope_mode = "--slope-mode"; will turn this on
$rrd_vrule_color    = "FF0000";

$hr_width           = $rrd_width + 130;

# Webpage Parameters

$bg_color           = "#F8F8F8"; 
$text_color         = "#000000";
$link_color         = "#006699";
$vlink_color        = "#BF294D";

# Commands (full directory names)
 
$dig                = "/usr/bin/dig +time=1 -x ";
